package ru.dbotthepony.kstarbound

import com.google.gson.JsonElement
import com.google.gson.JsonObject
import com.google.gson.JsonParser
import ru.dbotthepony.kstarbound.defs.TileDefinition
import ru.dbotthepony.kstarbound.defs.TileDefinitionBuilder
import ru.dbotthepony.kstarbound.defs.TileRenderTemplate
import ru.dbotthepony.kstarbound.world.World
import java.io.File
import java.io.FileNotFoundException

object Starbound {
	val tiles = HashMap<String, TileDefinition>()
	val world = World()

	private val _filepath = ArrayList<File>()
	val filepath = object : List<File> by _filepath {}

	fun addFilePath(path: File) {
		_filepath.add(path)
	}

	fun findFile(path: File): File {
		if (path.exists()) {
			return path.canonicalFile
		}

		for (sPath in _filepath) {
			val newPath = File(sPath.path, path.path)

			if (newPath.exists()) {
				return newPath
			}
		}

		throw FileNotFoundException("Unable to find $path in any of known file paths")
	}

	fun findFile(path: String) = findFile(File(path))

	fun loadJson(path: String): JsonElement {
		if (path[0] == '/')
			return JsonParser.parseReader(findFile(path.substring(1)).bufferedReader())

		return JsonParser.parseReader(findFile(path).bufferedReader())
	}

	fun loadTileDefinition(name: String): TileDefinition {
		return tiles.computeIfAbsent(name) {
			val foundPath = findFile("tiles/materials/$name.material")
			return@computeIfAbsent TileDefinitionBuilder.fromJson(JsonParser.parseReader(foundPath.bufferedReader()) as JsonObject).build(foundPath.parent)
		}
	}
}
